"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ArmManager = void 0;
var Q = require("q");
var utilities_1 = require("../Helpers/utilities");
var ArmManager;
(function (ArmManager) {
    "use strict";
    var mockLocations = [
        {
            "id": "/subscriptions/f873fabb-21aa-4f93-97b0-a5293389d19f/locations/eastasia",
            "displayName": "East Asia",
            "name": "eastasia",
            "latitude": 22.267,
            "longitude": 114.188
        },
        {
            "id": "/subscriptions/f873fabb-21aa-4f93-97b0-a5293389d19f/locations/southeastasia",
            "displayName": "Southeast Asia",
            "name": "southeastasia",
            "latitude": 1.283,
            "longitude": 103.833
        },
        {
            "id": "/subscriptions/f873fabb-21aa-4f93-97b0-a5293389d19f/locations/centralus",
            "displayName": "Central US",
            "name": "centralus",
            "latitude": 41.5908,
            "longitude": -93.6208
        },
        {
            "id": "/subscriptions/f873fabb-21aa-4f93-97b0-a5293389d19f/locations/eastus",
            "displayName": "East US",
            "name": "eastus",
            "latitude": 37.3719,
            "longitude": -79.8164
        },
        {
            "id": "/subscriptions/f873fabb-21aa-4f93-97b0-a5293389d19f/locations/eastus2",
            "displayName": "East US 2",
            "name": "eastus2",
            "latitude": 36.6681,
            "longitude": -78.3889
        },
        {
            "id": "/subscriptions/f873fabb-21aa-4f93-97b0-a5293389d19f/locations/westus",
            "displayName": "West US",
            "name": "westus",
            "latitude": 37.783,
            "longitude": -122.417
        },
        {
            "id": "/subscriptions/f873fabb-21aa-4f93-97b0-a5293389d19f/locations/northcentralus",
            "displayName": "North Central US",
            "name": "northcentralus",
            "latitude": 41.8819,
            "longitude": -87.6278
        },
        {
            "id": "/subscriptions/f873fabb-21aa-4f93-97b0-a5293389d19f/locations/southcentralus",
            "displayName": "South Central US",
            "name": "southcentralus",
            "latitude": 29.4167,
            "longitude": -98.5
        },
        {
            "id": "/subscriptions/f873fabb-21aa-4f93-97b0-a5293389d19f/locations/northeurope",
            "displayName": "North Europe",
            "name": "northeurope",
            "latitude": 53.3478,
            "longitude": -6.2597
        },
        {
            "id": "/subscriptions/f873fabb-21aa-4f93-97b0-a5293389d19f/locations/westeurope",
            "displayName": "West Europe",
            "name": "westeurope",
            "latitude": 52.3667,
            "longitude": 4.9
        },
        {
            "id": "/subscriptions/f873fabb-21aa-4f93-97b0-a5293389d19f/locations/japanwest",
            "displayName": "Japan West",
            "name": "japanwest",
            "latitude": 34.6939,
            "longitude": 135.5022
        },
        {
            "id": "/subscriptions/f873fabb-21aa-4f93-97b0-a5293389d19f/locations/japaneast",
            "displayName": "Japan East",
            "name": "japaneast",
            "latitude": 35.68,
            "longitude": 139.77
        },
        {
            "id": "/subscriptions/f873fabb-21aa-4f93-97b0-a5293389d19f/locations/brazilsouth",
            "displayName": "Brazil South",
            "name": "brazilsouth",
            "latitude": -23.55,
            "longitude": -46.633
        },
        {
            "id": "/subscriptions/f873fabb-21aa-4f93-97b0-a5293389d19f/locations/australiaeast",
            "displayName": "Australia East",
            "name": "australiaeast",
            "latitude": -33.86,
            "longitude": 151.2094
        },
        {
            "id": "/subscriptions/f873fabb-21aa-4f93-97b0-a5293389d19f/locations/australiasoutheast",
            "displayName": "Australia Southeast",
            "name": "australiasoutheast",
            "latitude": -37.8136,
            "longitude": 144.9631
        }
    ];
    /**
     * The mock azure resource manager
     */
    var Manager = /** @class */ (function () {
        function Manager() {
            this.data = {};
            this.events = {};
        }
        /**
         * Initializes the mock data. Call this with empty data to reset state.
         *
         * @param data The mock data.
         * @return A promise that is resolved when the data is initialized.
         */
        Manager.prototype.initializeMockData = function (data) {
            data = data || {};
            this.data = data;
            this.events = {};
            return Q.resolve(true);
        };
        /**
         * Gets a subscription based on id from the mock data.
         *
         * @param subscriptionId The subscription GUID/ID.
         */
        Manager.prototype.getEvents = function (subscriptionId) {
            return this.events[subscriptionId.toLowerCase()] || [];
        };
        /**
         * Gets the list of providers.
         */
        Manager.prototype.getProviders = function () {
            return this.data.providers || [];
        };
        /**
         * Gets list of mock locations.
         */
        Manager.prototype.getLocations = function () {
            return mockLocations;
        };
        /**
         * Gets the list of mock subscriptions.
         */
        Manager.prototype.getSubscriptions = function () {
            return (this.data.subscriptions || []).map(function (sub) {
                if (!sub.mappedData) {
                    sub.mappedData = {
                        id: "/subscriptions/" + sub.subscriptionId,
                        subscriptionId: sub.subscriptionId,
                        state: sub.state,
                        displayName: sub.displayName,
                        subscriptionPolicies: sub.subscriptionPolicies,
                    };
                }
                return sub.mappedData;
            });
        };
        /**
         * Gets a subscription based on id from the mock data.
         *
         * @param subscriptionId The subscription GUID/ID.
         */
        Manager.prototype.getSubscription = function (subscriptionId) {
            subscriptionId = subscriptionId.toLowerCase();
            return utilities_1.Utilities.findInArr(this.getSubscriptions(), function (sub) { return sub.subscriptionId.toLowerCase() === subscriptionId; });
        };
        /**
         * Gets the list of mock resource groups under a subscription.
         *
         * @param subscriptionId The subscription GUID/ID.
         */
        Manager.prototype.getResourceGroups = function (subscriptionId) {
            var subData = this.getSubscriptionData(subscriptionId);
            if (!subData) {
                return [];
            }
            return subData.resourceGroups.map(function (rg) {
                if (!rg.mappedData) {
                    rg.mappedData = {
                        id: "/subscriptions/" + subscriptionId + "/resourcegroups/" + rg.name,
                        name: rg.name,
                        location: rg.location,
                        tags: rg.tags,
                        properties: rg.properties,
                    };
                }
                return rg.mappedData;
            });
        };
        /**
         * Gets the resource group based on id from the mock data.
         *
         * @param subscriptionId The subscription GUID/ID.
         * @param resourceGroupId The resource group name.
         */
        Manager.prototype.getResourceGroup = function (subscriptionId, resourceGroupId) {
            resourceGroupId = resourceGroupId.toLowerCase();
            return utilities_1.Utilities.findInArr(this.getResourceGroups(subscriptionId), function (rg) { return rg.name.toLowerCase() === resourceGroupId; });
        };
        /**
         * Gets all resources under a subscription and resource group
         *
         * @param subscriptionId The subscription GUID/ID.
         * @param resourceGroupId The resource group name.
         */
        Manager.prototype.getResources = function (subscriptionId, resourceGroupId) {
            var rgData = this.getResourceGroupData(subscriptionId, resourceGroupId);
            if (!rgData) {
                return [];
            }
            var nestedResources = [];
            var resources = rgData.resources.map(function (r) {
                if (!r.mappedData) {
                    r.mappedData = {
                        id: "/subscriptions/" + subscriptionId + "/resourcegroups/" + rgData.name + "/providers/" + r.type + "/" + r.name,
                        name: r.name,
                        type: r.type,
                        location: r.location,
                        tags: r.tags,
                        resourceGroup: "/subscriptions/" + subscriptionId + "/resourcegroups/" + rgData.name,
                        kind: r.kind,
                        selfLinkUri: null,
                    };
                }
                if (r.nestedResources) {
                    nestedResources = nestedResources.concat(r.nestedResources.map(function (nr) {
                        if (!nr.mappedData) {
                            nr.mappedData = {
                                id: r.mappedData.id + "/" + nr.type + "/" + nr.name,
                                name: r.mappedData.name + "/" + nr.name,
                                type: r.mappedData.type + "/" + nr.type,
                                location: r.mappedData.location,
                                tags: nr.tags,
                                resourceGroup: r.mappedData.resourceGroup,
                                kind: null,
                                selfLinkUri: null,
                            };
                        }
                        return nr.mappedData;
                    }));
                }
                return r.mappedData;
            });
            return resources.concat(nestedResources);
        };
        /**
         * Gets a resource under a subscription and resource group.
         *
         * @param subscriptionId The subscription GUID/ID.
         * @param resourceGroupId The resource group name.
         * @param type The resource type.
         * @param resourceName The resource name.
         */
        Manager.prototype.getResource = function (subscriptionId, resourceGroupId, type, resourceName) {
            resourceName = resourceName.toLowerCase();
            type = type.toLowerCase();
            return utilities_1.Utilities.findInArr(this.getResources(subscriptionId, resourceGroupId), function (r) {
                return r.type.toLowerCase() === type && r.name.toLowerCase() === resourceName;
            });
        };
        /**
         * Publishes a subscription event.
         *
         * @param subscriptionId The subscription GUID/ID.
         * @param data The event data.
         */
        Manager.prototype.publishEvent = function (subscriptionId, data) {
            var subId = subscriptionId.toLowerCase();
            var subEvents = this.events[subId];
            if (!subEvents) {
                this.events[subId] = [];
            }
            this.events[subId].push(data);
        };
        Manager.prototype.getSubscriptionData = function (subscriptionId) {
            return utilities_1.Utilities.findInArr(this.data.subscriptions || [], function (sub) {
                return sub.subscriptionId.toLowerCase() === subscriptionId.toLowerCase();
            });
        };
        Manager.prototype.getResourceGroupData = function (subscriptionId, resourceGroupId) {
            var subData = this.getSubscriptionData(subscriptionId);
            if (!subData) {
                return null;
            }
            return utilities_1.Utilities.findInArr(subData.resourceGroups || [], function (rg) {
                return rg.name.toLowerCase() === resourceGroupId.toLowerCase();
            });
        };
        Manager.prototype.getResourceData = function (subscriptionId, resourceGroupId, type, resourceName) {
            var rgData = this.getResourceGroupData(subscriptionId, resourceGroupId);
            if (!rgData) {
                return null;
            }
            return utilities_1.Utilities.findInArr(rgData.resources || [], function (r) {
                return r.type.toLowerCase() === type.toLowerCase() && r.name.toLowerCase() === resourceName.toLowerCase();
            });
        };
        Manager.prototype.getNestedResourceData = function (subscriptionId, resourceGroupId, type, resourceName, nestedType, nestedResourceName) {
            var rData = this.getResourceData(subscriptionId, resourceGroupId, type, resourceName);
            if (!rData) {
                return null;
            }
            return utilities_1.Utilities.findInArr(rData.nestedResources || [], function (r) {
                return r.type.toLowerCase() === nestedType.toLowerCase() && r.name.toLowerCase() === nestedResourceName.toLowerCase();
            });
        };
        return Manager;
    }());
    ArmManager.Manager = Manager;
})(ArmManager = exports.ArmManager || (exports.ArmManager = {}));
//# sourceMappingURL=armManager.js.map