"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ArmProxy = void 0;
var armManager_1 = require("./armManager");
var batch_1 = require("./Controllers/batch");
var certHelper_1 = require("../Helpers/certHelper");
var firewallHelper_1 = require("../Helpers/firewallHelper");
var httpsProxy_1 = require("../HttpsProxy/httpsProxy");
var invoke_1 = require("./Controllers/invoke");
var locations_1 = require("./Controllers/locations");
var locks_1 = require("./Controllers/locks");
var notifications_1 = require("./Controllers/notifications");
var subscriptions_1 = require("./Controllers/subscriptions");
var providers_1 = require("./Controllers/providers");
var resources_1 = require("./Controllers/resources");
var resourceGroups_1 = require("./Controllers/resourceGroups");
var urlParams_1 = require("./Controllers/urlParams");
var ArmProxy;
(function (ArmProxy) {
    "use strict";
    var firewallRuleName = "Azure Portal ARM Test Proxy";
    /**
     * Creates an Arm proxy instance and setups firewal and certificates.
     *
     * @param armEndpoint The arm endpoint to proxy.
     * @param serverPort The local server port.
     * @param armManager The arm mock manager.
     * @param enablePassthrough If to enable passthrough to actual arm.
     * @return A promise that completes when local proxy is setup.
     */
    function create(armEndpoint, serverPort, armManager, customRoutes, enablePassthrough) {
        if (enablePassthrough === void 0) { enablePassthrough = true; }
        process.env.NODE_TLS_REJECT_UNAUTHORIZED = "0";
        armEndpoint = armEndpoint.replace("https://", "").replace("/", "");
        return firewallHelper_1.FirewallHelper.addFirewallRule(firewallRuleName, serverPort, true).then(function () {
            return certHelper_1.CertHelper.createLocalHostSSLOptions().then(function (sslOptions) {
                var serverOptions = {
                    serverPort: serverPort
                };
                var serverPromise;
                if (enablePassthrough) {
                    serverPromise = httpsProxy_1.HttpsProxy.createServerWithProxy(sslOptions, armEndpoint, serverOptions);
                }
                else {
                    serverPromise = httpsProxy_1.HttpsProxy.createServer(sslOptions, armEndpoint, serverOptions);
                }
                return serverPromise.then(function (server) {
                    server.app.armManager = armManager || new armManager_1.ArmManager.Manager();
                    // register user routes first
                    if (customRoutes)
                        customRoutes(server.app);
                    initializeRoutes(server.app);
                    // set the default routes, SHOULD be registered after all other routes as express matches routes
                    // from first registered to last.
                    server.initializeDefaultRoutes();
                    return server;
                });
            });
        });
    }
    ArmProxy.create = create;
    /**
     * Dispose local proxy
     *
     * @param armEndpoint The arm endpoint to proxy.
     * @param serverPort The local server port.
     * @param enablePassthrough If to enable passthrough to actual arm.
     * @return A promise that completes when local proxy is setup.
     */
    function dispose(proxy) {
        return firewallHelper_1.FirewallHelper.removeFirewallRule(firewallRuleName).then(function () {
            proxy.server.close();
            if (proxy.proxy)
                proxy.proxy.close();
        });
    }
    ArmProxy.dispose = dispose;
    function initializeRoutes(app) {
        urlParams_1.UrlParams.init(app);
        app.use("/api/invoke", invoke_1.Invoke.serverRouter);
        app.use("/batch", batch_1.Batch.serverRouter);
        subscriptions_1.Subscriptions.init(app);
        resourceGroups_1.ResourceGroups.init(app);
        resources_1.Resources.init(app);
        providers_1.Providers.init(app);
        locations_1.Locations.init(app);
        locks_1.Locks.init(app);
        notifications_1.Notifications.init(app);
    }
})(ArmProxy = exports.ArmProxy || (exports.ArmProxy = {}));
//# sourceMappingURL=armProxy.js.map