"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ODataParser = void 0;
var odataParser = require("odata-parser");
var ODataParser;
(function (ODataParser) {
    var opFuncMap = {
        and: and,
        or: or,
        eq: eq,
        functioncall: functioncall
    };
    /**
     * Evaluates the OData filter on the passed in object.
     *
     * @param filter The OData filter string e.g. $filter='subscriptionId eq 'sub1''.
     * @param obj The object array on which the filter is applied.
     * @return The filtered object array.
     */
    function evaluate(filter, obj) {
        var ast = odataParser.parse(filter);
        return obj.filter(function (val) { return runAST(ast.$filter, val); });
    }
    ODataParser.evaluate = evaluate;
    function runAST(ast, val) {
        if (ast) {
            var operator = ast.type;
            var operatorFunc = opFuncMap[operator];
            if (operatorFunc) {
                return operatorFunc(ast, val);
            }
            else {
                throw Error("Unknown operator. " + operator);
            }
        }
        return true;
    }
    function and(ast, val) {
        return runAST(ast.left, val) && runAST(ast.right, val);
    }
    function or(ast, val) {
        return runAST(ast.left, val) || runAST(ast.right, val);
    }
    function eq(ast, val) {
        var property = ast.left;
        var literal = ast.right;
        if (property.type !== "property" || literal.type !== "literal") {
            throw new Error("Unsupported 'eq' usage. " + JSON.stringify(ast));
        }
        return (getKeyAsString(property.name, val) + "").toLowerCase() === (literal.value + "").toLowerCase();
    }
    function functioncall(ast, val) {
        var func = ast.func;
        if (func === "substringof" && ast.args && ast.args.length === 2) {
            var literal = ast.args[0];
            var property = ast.args[1];
            if (property.type !== "property" || literal.type !== "literal") {
                throw new Error("Unsupported 'substringof' usage. " + JSON.stringify(ast));
            }
            return ((getKeyAsString(property.name, val) + "").toLowerCase()).indexOf((literal.value + "").toLowerCase()) >= 0;
        }
        else {
            throw Error("Unknown function. " + func);
        }
    }
    function getKeyAsString(key, val) {
        key = key.toLowerCase();
        var ret;
        Object.keys(val).forEach(function (prop) {
            if (key === prop.toLowerCase()) {
                ret = val[prop];
            }
        });
        return typeof ret === "string" ? ret : JSON.stringify(ret);
    }
})(ODataParser = exports.ODataParser || (exports.ODataParser = {}));
//# sourceMappingURL=odataHelper.js.map