Param(
    [Parameter(Mandatory=$true)]
    [string]$InDnsName="",
    
    [Parameter(Mandatory=$true)]
    [string]$InPassword=""
)

Add-Type -TypeDefinition 'public enum CertificateType { SSLServerAuthentication }'
Add-Type -TypeDefinition 'public enum KeyExportPolicy { Exportable }'

Function New-SelfSignedCertificate {
    Param(
        [Parameter(Mandatory=$true)]
        [string]$CertStoreLocation,

        [Parameter(Mandatory=$true)]
        [string]$DnsName,

        [Parameter(Mandatory=$true)]
        [DateTime]$NotBefore,

        [Parameter(Mandatory=$true)]
        [DateTime]$NotAfter,

        [Parameter(Mandatory=$true)]
        [CertificateType]$Type=[CertificateType]::SSLServerAuthentication,

        [Parameter(Mandatory=$true)]
        [int]$KeyLength=2048,

        [Parameter(Mandatory=$true)]
        [KeyExportPolicy]$KeyExportPolicy=[KeyExportPolicy]::Exportable,

        [Parameter(Mandatory=$true)]
        [string]$Provider="Microsoft RSA SChannel Cryptographic Provider"
    )

    $name = New-Object -com "X509Enrollment.CX500DistinguishedName.1"
    $name.Encode("CN=$DnsName", 0)

    $key = New-Object -com "X509Enrollment.CX509PrivateKey.1"
    $key.ProviderName = $Provider
    $key.KeySpec = 1
    $key.Length = $KeyLength
    $key.ExportPolicy = 1
    $key.SecurityDescriptor = "D:PAI(A;;0xd01f01ff;;;SY)(A;;0xd01f01ff;;;BA)(A;;0x80120089;;;NS)"
    $key.MachineContext = 1
    $key.Create()

    $cert = New-Object -com "X509Enrollment.CX509CertificateRequestCertificate.1"
    $cert.InitializeFromPrivateKey(2, $key, "")
    $cert.Subject = $name
    $cert.Issuer = $cert.Subject
    $cert.NotBefore = $NotBefore
    $cert.NotAfter = $NotAfter

    # Create Extended Key Usage extension
    $serverAuthOID = New-Object -com "X509Enrollment.CObjectId.1"
    $serverAuthOID.InitializeFromValue("1.3.6.1.5.5.7.3.1")
    $ekuOIDs = New-Object -com "X509Enrollment.CObjectIds.1"
    $ekuOIDs.add($serverAuthOID)
    $ekuExt = New-Object -com "X509Enrollment.CX509ExtensionEnhancedKeyUsage.1"
    $ekuExt.InitializeEncode($ekuOIDs)

    # Add extensions to the certificate request
    $cert.X509Extensions.Add($ekuExt)

    $cert.Encode()

    $enrollment = New-Object -com "X509Enrollment.CX509Enrollment.1"
    $enrollment.InitializeFromRequest($cert)
    $certdata = $enrollment.CreateRequest(0)
    $enrollment.InstallResponse(2, $certdata, 0, "")

    $pfxPassword = [System.Guid]::NewGuid().ToString()
    $pfxData = $enrollment.CreatePFX($pfxPassword, 2)
    $securePfxPassword = ConvertTo-SecureString -String $pfxPassword -Force -AsPlainText
    $pfxPath = [System.IO.Path]::GetTempFileName()
    Try {
        [System.IO.File]::WriteAllText($pfxPath, $pfxData);
        Import-PfxCertificate -FilePath $pfxPath $CertStoreLocation -Password $securePfxPassword -Exportable
    }
    Finally {
        [System.IO.File]::Delete($pfxPath)
    }
}

Function New-Certificate {
    Param(
        [Parameter(Mandatory=$true)]
        [string]$DnsName,

        [Parameter(Mandatory=$true)]
        [string]$Path,

        [Parameter(Mandatory=$true)]
        [string]$Password
    )
    $log = @()

    # Remove existing cert(s)
    $results = Get-ChildItem Cert:\LocalMachine\My | Where-Object { $_.Subject -eq "CN=$DnsName" }
    if ($results) {
        $log += 'Deleting old SSL cert(s) from \LocalMachine\My...'
        $results | %{Remove-Item -path $_.PSPath -recurse -Force}
    }
    $results = Get-ChildItem Cert:\LocalMachine\Root | Where-Object { $_.Subject -eq "CN=$DnsName" }
    if ($results) {
        $log += 'Deleting old SSL cert(s) from \LocalMachine\Root...'
        $results | %{Remove-Item -path $_.PSPath -recurse -Force}
    }
    $results = Get-ChildItem Cert:\LocalMachine\CA | Where-Object { $_.Subject -eq "CN=$DnsName" }
    if ($results) {
        $log += 'Deleting old SSL cert(s) from \LocalMachine\CA...'
        $results | %{Remove-Item -path $_.PSPath -recurse -Force}
    }

    $begin = Get-Date
    $end = $begin.AddDays(90)

    $sslCert = New-SelfSignedCertificate -CertStoreLocation Cert:\LocalMachine\My -DnsName $DnsName -NotBefore $begin -NotAfter $end -Type SSLServerAuthentication -KeyLength $KeyLength -KeyExportPolicy Exportable -Provider "Microsoft RSA SChannel Cryptographic Provider"

    if ($sslCert) {
        $log += 'Created SSL cert "$($sslCert.Subject) $($sslCert.Thumbprint)".'

        # Make browsers trust this new cert by copying it's public key to the Truste Root store
        # Because only the public key is copied the cert is single use and cannot be used to sign any other certs
        $certStore = New-Object System.Security.Cryptography.X509Certificates.X509Store 'Root', 'LocalMachine'
        $certStore.Open('ReadWrite')
        try {
            $sslCert | ForEach-Object { $certStore.Add($_) }
            $log += 'Added SSL cert "$($sslCert.Subject) $($sslCert.Thumbprint)" to "Root" store.'
        }
        finally {
            $certStore.Close();
        }

        $pwd = ConvertTo-SecureString -String $Password -Force -AsPlainText
        Export-PfxCertificate -FilePath $Path -Cert $sslCert -Password $pwd
    } else {
        $log += 'Failed to create ' + $DnsName + ' SSL cert'
    }

    return $log
}

$path = ([System.IO.Path]::GetTempFileName()) + '.pfx'
$log = New-Certificate -DnsName $InDnsName -Path $path -Password $InPassword
$log

return $path