/// <reference types="node" />
import { Request as ExpressRequest, Router } from "express";
import { IncomingMessage } from "http";
import { LoggerType } from "./logger";
import { MockHeaders as MockHeaders } from "./server";
/**
 * Used internally to verify all the optional Context properties have been assigned
 * default values and adds any extra internal properties
 */
export interface _FullContext extends Required<MockFx.Context> {
    currentStage: string;
}
export declare class MockFx {
    /**
     * The current version of the framework. It's a good idea to use this to bin your stored mock data
     * to allow for upgrading the framework when breaking changes are introduced in order to not break
     * test runs using previous version.
     *
     * @static
     * @memberof MockFx
     */
    static version: string;
    /**
     * Log event emitter. May subscribe to "log", "telemetry" and "error" events.
     *
     * @static
     * @type {LoggerType}
     * @memberof MockFx
     */
    static logger: LoggerType;
    /**
     * Creates an instance of the framework
     *
     * @static
     * @param {MockFx.Configuration} config
     * @returns {Promise<MockFx>}
     * @memberof MockFx
     */
    static create(config: MockFx.Configuration): Promise<MockFx>;
    /**
     * Returns an object with endpoints for the mock server for registering, finalizing, and obtaining the plug-in
     * code for a test.
     *
     * If using the plug-in endpoint with Portal's `patch` parameter, run the endpoint value through `encodeURI`
     *
     * @param {string} runId
     * @param {string} testId
     * @returns {MockFx.Endpoints}
     * @memberof MockFx
     */
    static getEndpoints(port: number, runId: string, testId: string): MockFx.Endpoints;
    private static loggingToConsole;
    private static logToConsole;
    /**
     * Starts the mock server
     *
     * @returns {Promise<void>}
     * @memberof MockFx
     */
    start(): Promise<void>;
    /**
     * Stops the mock server
     *
     * @returns {Promise<void>}
     * @memberof MockFx
     */
    stop(): Promise<void>;
    /**
     * Finalizes a stage within a test. During replay, removes any subsequent requests that were tagged with this stage
     *
     * Also accessible over HTTPS POST at `MockFx.getEndpoints(mockFxPort, runId, testId).finalizeStage` end-point
     * by sending the stage context object as the body
     *
     * @param {MockFx.Stage} stageContext
     * @returns {Promise<void>}
     * @memberof MockFx
     */
    finalizeStage(stageContext: MockFx.Stage): Promise<void>;
    /**
     * Calls `storeRequests` on each target the test has registered, then removes the references to the instance
     * to free memory.
     *
     * Also accessible over HTTPS POST at `MockFx.getEndpoints(mockFxPort, runId, testId).finalizeTest` end-point
     * by sending the finalization object as the body
     *
     * @param {MockFx.Finalization} finalization
     * @returns {Promise<MockFx.Context>}
     * @memberof MockFx
     */
    finalizeTest(finalization: MockFx.Finalization): Promise<MockFx.Context>;
    /**
     * Returns the mock plug-in client code to run in the browser
     *
     * Also accessible over HTTPS GET at `MockFx.getEndpoints(mockFxPort, runId, testId).plugin` end-point
     *
     * @param {string} runId
     * @param {string} testId
     * @returns {string}
     * @memberof MockFx
     */
    getPluginCode(runId: string, testId: string): string;
    /**
     * Registers a stage within a test. All requests occurring for a test will be tagged with the current stage.
     * Used if parts of a test (a stage) are conditional or can be skipped.
     *
     * Also accessible over HTTPS POST at `MockFx.getEndpoints(mockFxPort, runId, testId).registerStage` end-point
     * by sending the stage context object as the body
     *
     * @param {MockFx.Stage} stageContext
     * @returns {Promise<void>}
     * @memberof MockFx
     */
    registerStage(stageContext: MockFx.Stage): Promise<void>;
    /**
     * Registers a new test, creates instances for each target referenced, and loads their test data if in
     * replay mode.
     *
     * Also accessible over HTTPS POST at `MockFx.getEndpoints(mockFxPort, runId, testId).registerTest` end-point
     * by sending the context object as the body
     *
     * @param {MockFx.Context} context
     * @returns {Promise<MockFx.Context>}
     * @memberof MockFx
     */
    registerTest(context: MockFx.Context): Promise<MockFx.Context>;
    private app;
    private certificate;
    private port;
    private proxies;
    private server?;
    private targetDefinitions;
    private tests;
    private constructor();
    private getTestInstance;
    private registerBaseRoutes;
    private registerPluginRoute;
    private registerErrorHandler;
    private registerFinalizeRoute;
    private registerFinalizeStageRoute;
    private registerRegisterRoute;
    private registerRegisterStageRoute;
    private registerRequestHandler;
    private handleProxyResponse;
    private normalizeHost;
}
export declare namespace MockFx {
    const enum Mode {
        Record = "record",
        Replay = "replay"
    }
    const enum LogLevel {
        Debug = "debug",
        Info = "info",
        Warn = "warn"
    }
    const enum Events {
        Error = "error",
        FinalizeStage = "finalizeStage",
        FinalizeTest = "finalizeTest",
        Log = "log",
        RegisterStage = "registerStage",
        RegisterTest = "registerTest",
        Telemetry = "telemetry"
    }
    export import Headers = MockHeaders;
    interface Configuration {
        /**
         * Send MockFx logs to the console in addition to the log event emitter.
         *
         * Setting this to true will affect the MockFx class and send logs to the console for all
         * created MockFx instances.
         *
         * @default false
         */
        logToConsole?: boolean;
        /**
         * The port for the server to listen on
         */
        port: number;
        /**
         * A list of target services to be mocked. Any target which a test wants to mock must be defined here.
         */
        targets: TargetDefinition[];
    }
    /**
     * @swagger
     * definitions:
     *   Context:
     *     required:
     *       - mode
     *       - runId
     *       - testId
     *       - targets
     *     properties:
     *       mode:
     *         description: "The mock mode of operation"
     *         type: string
     *         enum:
     *         - "replay"
     *         - "record"
     *       runId:
     *         description: "The run ID for the test"
     *         type: string
     *       testId:
     *         description: "The test ID for the test"
     *         type: string
     *       targets:
     *         description: "A list of target names to mock"
     *         type: "array"
     *         items:
     *           type: "string"
     *       metadata:
     *         description: "Any additional metadata to associate with the test."
     *         type: object
     *       proxyUnmatched:
     *         description: "Whether to send unmatched requests to the target during replay mode."
     *         type: boolean
     *         default: false
     */
    interface Context {
        /**
         * Whether to record or replay for this test
         */
        mode: MockFx.Mode;
        /**
         * A unique ID to group a set of running tests together
         */
        runId: string;
        /**
         * A unique ID for a single test instance within a run
         */
        testId: string;
        /**
         * A list of target names to mock during this test run (e.g. ["ARM"]).
         *
         * Target must have been previously defined in the MockFx instance.
         */
        targets: string[];
        /**
         * Provide any additional information about the test that you require.
         */
        metadata?: any;
        /**
         * Set this to true to send unmatched requests to the target while in replay mode, otherwise
         * an error response will be returned if a request cannot be matched
         *
         * Default: false
         */
        proxyUnmatched?: boolean;
    }
    /**
     * Returned from `MockFx.getEndpoints(...)`. Use these end-points to register, load client
     * plug-in code for, and finalize your test runs
     */
    interface Endpoints {
        /**
         * The stage finalization end-point
         */
        finalizeStage: string;
        /**
         * The test finalization end-point
         */
        finalizeTest: string;
        /**
         * The end-point for obtaining the client-side mock plug-in
         */
        plugin: string;
        /**
         * The stage registration end-point
         */
        registerStage: string;
        /**
         * The test registration end-point
         */
        registerTest: string;
    }
    /**
     * @swagger
     * definitions:
     *   Finalization:
     *     required:
     *       - runId
     *       - testId
     *     properties:
     *       runId:
     *         description: "The run ID for the test"
     *         type: string
     *       testId:
     *         description: "The test ID for the test"
     *         type: string
     *       shouldStore:
     *         description: "Whether to call storeRequests for the test."
     *         type: boolean
     *         default: true
     */
    interface Finalization {
        /**
         * The runId for a registered test
         */
        runId: string;
        /**
         * The testId for a registered test
         */
        testId: string;
        /**
         * Whether or not the storeRequests target definition method should be called for this test run
         * (i.e. finalizing a failed test without storing its mock data)
         *
         * Default: true
         */
        shouldStore?: boolean;
    }
    /**
     * @swagger
     * definitions:
     *   Stage:
     *     required:
     *       - runId
     *       - testId
     *       - stage
     *     properties:
     *       runId:
     *         description: "The run ID of the test"
     *         type: string
     *       testId:
     *         description: "The test ID of the test"
     *         type: string
     *       stage:
     *         description: "The stage name"
     *         type: string
     */
    interface Stage {
        /**
         * The runId for a registered test
         */
        runId: string;
        /**
         * The testId for a registered test
         */
        testId: string;
        /**
         * The name of the stage
         */
        stage: string;
    }
    interface TargetDefinition {
        /**
         * A unique name to identity the targeted service
         *
         * @example ARM
         */
        name: string;
        /**
         * An array of additional hostnames used to correlate requests to a target. If an incoming request's
         * host includes any of the strings in this array, it will be redirected to the framework.
         *
         * May include partial hostnames
         *
         * @example ["management.azure.com"] or [".blob.core.windows.net", ".file.core.windows.net"]
         */
        hosts: string[];
        /**
         * *Record mode*
         *
         * Called after an intercepted request has completed to provide any metadata to save as the MockRequest's
         * metadata property.
         *
         * @param {Express.Request} request - the request that was sent to the target
         * @param {IncomingMessage} response - the response from the target
         * @returns {object} the metadata object to set on the request
         */
        getRequestMetadata?: (request: ExpressRequest, response: IncomingMessage) => Promise<Record<string, any>>;
        /**
         * Used to provide a custom Express router to handle special target routing (e.g. ARM batch calls)
         */
        getRouter?: () => Router;
        /**
         * Called during test initialization to allow the target to preload any data necessary using the test context.
         *
         * @param {Context} context - the registered mock context object for the currently running test
         */
        onRegister?: (context: Context) => Promise<void>;
        /**
         * *Replay mode*
         *
         * Called during test initialization in replay mode to obtain the array of mocked requests to replay
         *
         * @param {Context} context - the registered mock context object for the currently running test
         * @returns Promise<MockRequest[]>
         */
        loadRequests: (context: Context) => Promise<MockFx.Request[]>;
        /**
         * *Replay mode*
         *
         * Called on incoming request to match it against the stored requests
         *
         * @param {Express.Request} request - the incoming request
         * @param {MockFx.Request[]} unmatchedRequests - the unmatched stored mock requests
         * @param {MockFx.Request[]} storedRequests - the stored mock requests
         * @returns {MockFx.Request | undefined} the matched request or nothing if no match was found
         */
        matchRequest: (request: ExpressRequest, unmatchedRequests: MockFx.Request[], storedRequests: MockFx.Request[], testContext?: MockFx.Context) => Promise<MockFx.Request | undefined>;
        /**
         * *Replay mode*
         *
         * Called before an incoming request has been matched to allow for modifying the stored requests during
         * test execution.
         *
         * Useful when needing to normalize requests which contain dynamic data that changes between runs.
         *
         * @param {Express.Request} request - the incoming request
         * @param {MockFx.Context} context - the registered mock context object for the currently running test
         * @param {MockFx.Request[]} storedRequests - the stored mock requests
         * @returns {MockFx.Request[]} a new array of mock requests which will replace the current stored requests
         */
        onBeforeRequestMatch?: (request: ExpressRequest, context: MockFx.Context, storedRequests: MockFx.Request[]) => Promise<MockFx.Request[]>;
        /**
         * *Record mode*
         *
         * Called before an incoming request has been sent to allow for returning a predefined response during test execution.
         *
         * Useful when using static mocks.
         *
         * @param {Express.Request} request - the incoming request
         * @param {MockFx.Context} context - the registered mock context object for the currently running test
         * @returns {MockFx.Request} a new mock request which contains the response data for the associated request, or null to send the request to the server
         */
        onBeforeRequestSent?: (request: ExpressRequest, context: Context) => Promise<MockFx.Request | undefined>;
        /**
         * *Record mode or with `Context.proxyUnmatched`*
         *
         * Called when processing target response. The value returned will be set as the response property
         * on the stored request.
         *
         * If not implemented, the response body will be converted to a UTF-8 encoded string.
         *
         * @param {IncomingMessage} response - the target response
         * @param {Buffer} responseBody - the body of the response
         * @returns {any}
         */
        parseResponse?: (response: IncomingMessage, responseBody: Buffer) => Promise<any>;
        /**
         * *Record mode*
         *
         * Called once the running test is finalized
         *
         * @param {MockFx.Context} context - the registered mock context object for the currently running test
         * @param {MockFx.Request[]} requests - the recorded requests
         */
        storeRequests: (context: MockFx.Context, requests: MockFx.Request[]) => Promise<void>;
    }
    interface Request {
        /**
         * Index of this request in the requests array
         */
        index?: number;
        /**
         * The URL for the request
         */
        url: string;
        /**
         * The request's HTTP method
         */
        method: string;
        /**
         * The current stage of the running test
         */
        stage?: string;
        /**
         * Used by targets to store any additonal data about the request from the target definition's
         * `getRequestMetadata` method
         */
        metadata: Record<string, any>;
        /**
         * The request body (if present)
         */
        request: any;
        /**
         * The response from the target. Value is returned from the target definition's `parseResponse`
         * method
         */
        response: any;
        /**
         * Any headers set on the response.
         */
        responseHeaders: Record<string, string>;
        /**
         * The size of the response in bytes.
         */
        responseSize: number;
        /**
         * The HTTP response code
         */
        responseCode: number;
    }
    interface Telemetry {
        /**
         * The test context for this request
         */
        context: MockFx.Context;
        /**
         * The source of the request (ARM client, Portal, etc.)
         */
        source: string;
        /**
         * The name of the matched target
         */
        target: string;
        /**
         * The request's original host
         */
        originalHost: string;
        /**
         * When the request was processed
         */
        time: string;
        /**
         * The URL of the request
         */
        url: string;
        /**
         * HTTP method for the request
         */
        method: string;
        /**
         * Whether the target matched the incoming request
         */
        isMatch: boolean;
        /**
         * Index of the matched request in the requests array
         */
        matchIndex?: number;
        /**
         * The size of the response received from the target
         */
        responseSizeInBytes: number;
        /**
         * Any additional metadata a target wishes to add to the telemetry
         */
        metadata?: Record<string, string>;
        /**
         * The reason for the failure to match
         */
        error?: string;
    }
}
