"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.logErrorIfRequestsDifferOnlyInApiVersion = exports.urlsAreSameApartFromApiVersion = void 0;
var logger_1 = require("../logger");
var apiVersionRegex = /api-version=(.*?)(?:&|$)/;
/**
 * Tests whether two urls have different api versions but are otherwise identical, e.g.
 * urlsAreSameApartFromApiVersion("sameUrl?api-version=2021-10-01", "sameUrl?api-version=2019-09-01") returns true;
 * urlsAreSameApartFromApiVersion("sameUrl?api-version=2021-10-01", "sameUrl?api-version=2021-10-01") returns false.
 *
 * @param url1
 * @param url2
 * @returns
 */
function urlsAreSameApartFromApiVersion(url1, url2) {
    if (url1 === url2) {
        return false;
    }
    var url1ApiVersion = (url1.match(apiVersionRegex) || [])[1];
    var url2ApiVersion = (url2.match(apiVersionRegex) || [])[1];
    var url1WithoutApiVersion = url1.replace(apiVersionRegex, '');
    var url2WithoutApiVersion = url2.replace(apiVersionRegex, '');
    return url1ApiVersion && url2ApiVersion && url1ApiVersion !== url2ApiVersion && url1WithoutApiVersion === url2WithoutApiVersion;
}
exports.urlsAreSameApartFromApiVersion = urlsAreSameApartFromApiVersion;
/**
 * Logs error when urls of mock request and proxied request have different api versions but are otherwise the same
 *
 * @param url1
 * @param url2
 * @param partialMessage provide additional information about requests/matching
 * @returns
 */
function logErrorIfRequestsDifferOnlyInApiVersion(url1, url2, partialMessage, context) {
    if (urlsAreSameApartFromApiVersion(url1, url2)) {
        var error = new Error("Found a recorded request that differs only in ARM API version: current URL is \"".concat(url1, "\". Found recording is \"").concat(url2, "\". If updating API versions, make sure to update the mock recordings. ").concat(partialMessage));
        logger_1.logger.error(error, context);
        throw error;
    }
}
exports.logErrorIfRequestsDifferOnlyInApiVersion = logErrorIfRequestsDifferOnlyInApiVersion;
//# sourceMappingURL=helpers.js.map